/*******************************************************************************
 * sched_man.c
 * This file is part of "Assignment 1: Scheduling Simulator" 
 * Copyright (C) 2008 - c506001 (email: c506001@cse.hcmut.edu.vn )
 *
 * Content:
 * -------- 
 * Provide functions of managing Schedulers and Tasks
 *
 * System and Networking Department
 * Faculty of Computer Science and Engineering
 * Ho Chi Minh City University of Technology
 ******************************************************************************/
 
 
#include <stdio.h> 
#include <unistd.h>
#include <string.h>
#include "sched_man.h"

/**
 * Function name: freeTaskList
 * Description:
 *    Free TaskList's memory
 * Return value:
 *    0                : If sucessful
 *    ERR_TASKLIST_NULL: If TaskList is NULL
 **/
int freeTaskList(struct TaskList *tasks)
{
  //TODO: Free tasks

  return 0;
}

/**
 * Function name: freeSchedList
 * Description:
 *    Free SchedList's memory
 * Return value:
 *    0                 : If sucessful
 *    ERR_SCHEDLIST_NULL: If SchedList is NULL
 **/
int freeSchedList(struct SchedList *scheds)
{
  //TODO: Free scheds

  return 0;
}

/**
 * Function name: initSchedList
 * Description:
 *     Initialize default values of SchedList
 * Return value:
 *    0                 : If sucessful
 *    ERR_SCHEDLIST_NULL: If SchedList is NULL
 **/
int initSchedList(struct SchedList *schedList)
{
  if (schedList == NULL)
    return ERR_SCHEDLIST_NULL;

  freeSchedList(schedList);
  
  schedList->list = NULL;
  schedList->nscheds = 0;

  return 0;
}

/**
 * Function name: addScheduler
 * Description:
 *     Add Scheduler to the tail of the SchedList
 *     Set the state of Scheduler to inactive
 * Return value:
 *    0                  : If sucessful
 *    ERR_SCHEDLIST_NULL : If SchedList is NULL
 **/
int addScheduler(struct SchedList *schedList, struct Scheduler *sched)
{
  if (schedList == NULL)
    return ERR_SCHEDLIST_NULL;

  //TODO: Add sched to tail of schedList

  schedList->nscheds++;
  sched->state = SCHED_INACTIVE;
  return 0;
}

/**
 * Function name: setActiveSched
 * Description:
 *     Set a scheduler to be active
 * Return value:
 *    0                  : If sucessful
 *    ERR_SCHEDLIST_NULL : If SchedList is NULL
 *    ERR_SCHEDLIST_EMPTY: If SchedList is empty
 **/
int setActiveSched(struct SchedList *schedList, int schedId)
{
  struct Scheduler *sched;

  if (!schedList)
    return ERR_SCHEDLIST_NULL;

  sched = schedList->list;
  if (!sched)
    return ERR_SCHEDLIST_EMPTY;

  //TODO: Set state of Scheduler having its id equal to schedId to be active

  return 0;
}

/**
 * Function name: getActiveSched
 * Description:
 *     Find active scheduler in schedList and assign it to sched
 * Return value:
 *    SUCCESS           : If sucessful
 *    ERR_SCHEDLIST_NULL: If SchedList is NULL
 *    NO_ACTIVE_SCHED   : If there is no active scheduler
 **/
int getActiveSched(struct SchedList *schedList, struct Scheduler **sched)
{
  if (!schedList)
    return ERR_SCHEDLIST_NULL;

  //TODO: Find through schedList to get active scheduler
  //      Assign active scheduler to *sched

}

/**
 * Function name: addTailTaskList
 * Description:
 *     Add a new Task to the tail of a TaskList
 * Return value:
 *    0                : If sucessful
 *    ERR_TASKLIST_NULL: If TaskList is NULL
 **/
int addTailTaskList(struct TaskList *tasks, struct Task *t)
{
  if (!tasks)
    return ERR_TASKLIST_NULL;
    
  //TODO: Add t to tasks

  tasks->ntasks++;
  return 0;
}

/**
 * Function name: delTaskOutOfList
 * Description:
 *    Remove (unlink) a Task out of a TaskList
 * Return value:
 *    0                 : If sucessful
 *    ERR_TASKLIST_NULL : If TaskList is NULL
 *    ERR_TASKLIST_EMPTY: If TaskList is empty
 **/
int delTaskOutOfList(struct TaskList *tasks, struct Task *t)
{
  struct Task *next = t->next;
  struct Task *prev = t->prev;
  
  if (!tasks)
    return ERR_TASKLIST_NULL;
    
  if (!tasks->list)
    return ERR_TASKLIST_EMPTY;
    
  if (tasks->ntasks <= 1) {
    tasks->list = NULL;
  } else {
    if (tasks->list == t)
      tasks->list = next;
    next->prev = prev;
    prev->next = next;
  }

  tasks->ntasks--;
  return 0;
}

/**
 * Function name: sortTaskListByStartTime
 * Description:
 *    Sort task list by its start time
 *    You do not need to implement this function
 * Return value:
 *    0                : If sucessful
 *    ERR_TASKLIST_NULL: If TaskList is NULL
 **/
int sortTaskListByStartTime(struct TaskList *tasks)
{
  return 0;
}

/**
 * Function name: submitTask
 * Description:
 *     Submit satisfied tasks in TaskList to scheduler (queueingTasks)
 * Return value:
 *    SUCCESS            : If sucessful
 *    ALL_TASKS_SUBMITTED: If all tasks has been submitted
 *    ERR_SUBMIT_FAILED  : If error in submitting
 **/
int submitTask(struct TaskList *tasks, struct Scheduler *sched)
{
  struct Task *t;
  
  if ((!tasks) || (!sched)) 
    return ERR_SUBMIT_FAILED;
  
  t = tasks->list;
  if ((!t) || t->state == STATE_FINISHED) 
    return ALL_TASKS_SUBMITTED;

  //TODO: Find a task that has its start time <= sched->timer and 
  //        its state is not starting from tasks.
  //     Move that task to the tail of sched->queueingTasks
  
  return SUCCESS;
}

/**
 * Function name: delFinishedTask
 * Description:
 *    Delete finished task out of Schduler (queueingTasks)
 *    Push finished task to tail of outqueueTasks
 * Return value:
 *    0              : If sucessful
 *    ERR_SCHED_NULL : If Scheduler is NULL
 **/
int delFinishedTask(struct Scheduler *sched, struct Task *t)
{
  if (!sched)
    return ERR_SCHED_NULL;
    
  //TODO: Move finished task t to tail of sched->outqueueTasks

  return 0;
}
