/*******************************************************************************
 * sched_man.h
 * This file is part of "Assignment 1: Scheduling Simulator" 
 * Copyright (C) 2009 - c506001 (email: c506001@cse.hcmut.edu.vn )
 *
 * Content:
 * -------- 
 * Structures of Scheduler and Task.
 *
 * System and Networking Department
 * Faculty of Computer Science and Engineering
 * Ho Chi Minh City University of Technology
 ******************************************************************************/
 
 
#ifndef _SCHED_MAN_
#define _SCHED_MAN_

#define SCHED_ACTIVE            0
#define SCHED_INACTIVE          1

#define NO_ACTIVE_SCHED         ( 1)
#define SUCCESS                 ( 0)
#define ERR_TASK_NULL           (-1)
#define ERR_TASK_NOTNULL        (-2)
#define ERR_TASKLIST_NULL       (-3)
#define ERR_TASKLIST_NOTNULL    (-4)
#define ERR_TASKLIST_EMPTY      (-5)
#define ERR_SCHED_NULL          (-6)
#define ERR_SCHED_NOTNULL       (-7)
#define ERR_SCHEDLIST_NULL      (-8)
#define ERR_SCHEDLIST_NOTNULL   (-9)
#define ERR_SCHEDLIST_EMPTY     (-10)
#define ERR_SUBMIT_FAILED       (-11)
#define ERR_MALLOC_FAILED       (-12)
#define ERR_FILEOPEN_FAILED     (-13)

#define STATE_NOT_STARTING      0
#define STATE_RUNNING           1
#define STATE_SLEEPING          2
#define STATE_FINISHED          3

#define ALL_TASKS_SUBMITTED     1
#define NO_TASK                 2
#define NO_TASK_SUBMITTED	3

#define RR_SCHED_ID             3
#define SJF_SCHED_ID		5

/**
 * Struct name: Task
 * Description:
 *     Contain process' properties
 **/
struct Task {
    struct Task *next;
    struct Task *prev;
    int pid;
    int state;       // sleeping, running, not-starting, finished
    int startTime;   // the first time this task is processed by scheduler
    int lastTime;    // the last time this start is processed by scheduler
    int duration;
    int prio;
};

/**
 * Struct name: TaskList
 * Description:
 *     Manage a list of tasks
 **/
struct TaskList {
  struct Task *list;
  int ntasks;  // number of tasks in task list
};

/**
 * Struct name: Scheduler
 * Description:
 *     Contain scheduler's properties
 **/
struct Scheduler {
  struct Scheduler *next;
  struct Scheduler *prev;
  char name[32];                  // name of scheduler
  char description[1024];         // full description of scheduler
  int id;                         // id of scheduler
  struct TaskList queueingTasks;  // contain tasks being ready for running
  struct TaskList outqueueTasks;  // contain waiting or finished tasks 
  int state;                      // active or inactive
  int timer;                      // current time
  int (*run)(struct Scheduler *thisSched, char* outFile);
  int (*parseTasksFromFile)(struct Scheduler *thisSched, char *inFile);
};

/**
 * Struct name: SchedList
 * Description:
 *     Manage a list of schedulers
 **/
struct SchedList {
  struct Scheduler *list;
  int nscheds;  // number of schedulers in scheduler list
};

// functions declared and implemented in sched_man.c
extern int freeTaskList(struct TaskList *tasks);
extern int freeSchedList(struct SchedList *scheds);
extern int initSchedList(struct SchedList *schedList);
extern int addScheduler(struct SchedList *schedList, struct Scheduler *sched);
extern int setActiveSched(struct SchedList *schedList, int schedId);
extern int getActiveSched(struct SchedList *schedList, struct Scheduler **sched);
extern int addTailTaskList(struct TaskList *tasks, struct Task *t);
extern int delTaskOutOfList(struct TaskList *tasks, struct Task *t);
extern int submitTask(struct TaskList *tasks, struct Scheduler *sched);
extern int sortTaskListByStartTime(struct TaskList *tasks);
extern int sortTaskListByPriority(struct TaskList *tasks);
extern int delFinishedTask(struct Scheduler *sched, struct Task *t);
#endif
