/*******************************************************************************
 * sched.c
 * This file is part of "Assignment 1: Scheduling Simulator" 
 * Copyright (C) 2009 - c506001 (email: c506001@cse.hcmut.edu.vn )
 *
 * Content:
 * -------- 
 * Implement the main functions of RR scheduler
 *
 * System and Networking Department
 * Faculty of Computer Science and Engineering
 * Ho Chi Minh City University of Technology
 ******************************************************************************/
 
#include <stdio.h> 
#include <unistd.h>
#include <string.h>
#include "sched.h"
#include "task_man.h"

FILE * oFile;
int quantum;      // quantum time
int contextTime;  // remained time for context changing

/**
 * Function name: readConfFile
 * Description:
 *    Read config file to get the quantum time
 *    Store it to quantum variable
 *    If quantum<0, set it to 1
 * Return value:
 *    0                  : If sucessful
 *    ERR_FILEOPEN_FAILED: If can not open the config file
 **/
static int readConfFile(){
  FILE *confFile=fopen("scheds/rr/sched.conf","r");
  if(!confFile)
    return ERR_FILEOPEN_FAILED;

  //TODO: Read config file and put quantum value 
  // into the appropriate variable

  fclose(confFile);
  
  if (quantum<0) quantum=1;
  
  return 0;
}

/**
 * Function name: getRunningTask
 * Description:
 *    Find the running task
 * Return value:
 *    0              : If sucessful
 *    ERR_SCHED_NULL : If Scheduler is NULL
 **/
static int getRunningTask(struct Scheduler *sched, struct Task **t)
{
  if (!sched)
    return ERR_SCHED_NULL;

  //TODO: Find the task having smallest start time in queueingTasks
  // or the task that is in running state

  return 0;
}

/**
 * Function name: selectTask
 * Description:
 *    Select a running task and simulate its running
 *    The main scheduling algorithm implemented here
 * Return value:
 *    0             : If sucessful
 *    NO_TASK       : If there is no task in queueing list
 *    ERR_SCHED_NULL: If Scheduler is NULL
 **/
static int selectTask(struct Scheduler *sched)
{
  struct Task *runningTask; 

  if (!sched)
    return ERR_SCHED_NULL;
      
  //TODO: Complete missing parts
  
  //get running task
  getRunningTask(sched,&runningTask);
  
  // Update task's information
  runningTask->state = STATE_RUNNING;
  runningTask->lastTime = sched->timer;
  runningTask->duration--;
  
  if (runningTask->duration <= 0) {
    // Remove finished task
    delFinishedTask(sched, runningTask);        

    // Reset context time
  } else {

    //TODO: calculate context time for running task and 
    // switch context if the current task has run out of
    // context time

    }
  }

  return 0;
}

/**
 * Function name: runRR
 * Description:
 *    Simulate the RR Scheduler
 * Return value:
 *    0                  : If sucessful
 *    ERR_FILEOPEN_FAILED: If can not open the config file or ouput file 
 *    ERR_SCHED_NULL     : If Scheduler is NULL
 **/
static int runRR(struct Scheduler *thisSched, char* outFile)
{
  int submitFlag;
  int selectFlag;
  thisSched->timer = 0;
  
  printf("You are running RR SCHEDULER\n");
  
  // Read config file
  if (readConfFile()!=0) return ERR_FILEOPEN_FAILED;
  contextTime=quantum;
  
  // Open output file for writing
  oFile = fopen (outFile,"w");
  if(!oFile)
    return ERR_FILEOPEN_FAILED;

  // do scheduling
  while (1) {
    // submit tasks to scheduler (from outqueue to queueing)
    submitFlag = submitTask(&thisSched->outqueueTasks, thisSched);
    // select task from queue to execute
    selectFlag = selectTask(thisSched);

    if ((submitFlag == ALL_TASKS_SUBMITTED) && (selectFlag == NO_TASK))
      break;
    
    // timer run
    thisSched->timer++;
  }
  
  // Close output file
  fclose (oFile);

  return 0;
}

/**
 * Function name: parseTasksFromFileRR
 * Description:
 *    Create outqueue task list from input file
 * Return value:
 *    Return value of parseRRTaskFile function
 **/
static int parseTasksFromFileRR(struct Scheduler *thisSched, char *inFile)
{
  return parseRRTaskFile(&thisSched->outqueueTasks, inFile);
}

/**
 * Function name: initRRSched
 * Description:
 *    Initialize default values of RR Scheduler
 * Return value:
 *    0: If successful
 **/
int initRRSched(struct Scheduler *sched)
{
  //TODO: Initialize FIFO scheduler with default values

  return 0;
}
