/*******************************************************************************
 * schedsim.c
 * This file is part of "Assignment 1: Scheduling Simulator" 
 * Copyright (C) 2009 - c506001 (email: c506001@cse.hcmut.edu.vn )
 *
 * Content:
 * -------- 
 * The Main Program
 *
 * System and Networking Department
 * Faculty of Computer Science and Engineering
 * Ho Chi Minh City University of Technology
 ******************************************************************************/
 
 
#include <stdlib.h>
#include <stdio.h> 
#include <unistd.h>
#include <string.h>
#include "sched_man.h"
#include "schedsim.h"

static struct SchedList scheds;

static struct Options opts = {
  //TODO: Set default variables for opts
};

/**
 * Function name: parseArguments
 * Description:
 *    Determine what options or arguments user used
 *    Command-line options:
 * 	 [-h]         : Show help information
 *	 [-s] num     : Choose the active scheduler
 *	 [-i] filename: Choose the input file
 *	 [-o] filename: Choose the output file
 *	 [-l]         : Show all schedulers
 * Return value:
 *    0: If parsing successful
 *    exit: If error
 **/
static int parseArguments(int argc, char **argv, struct Options *options)
{
  int opt;
  extern char * optarg;    // (global variable) command-line options

  while ((opt = getopt(argc, argv, "")) != EOF)
  {
    switch (opt) {

      //TODO: Process options in command line

      default:
        fprintf(stderr, "Try 'schedsim -h' for more information\n");
        exit(1);
    }
  }

  return 0;
}

/**
 * Function name: Usage
 * Description:
 *    Show helping information when user uses -h option
 **/
static void Usage(){
  printf("SYNOPSIS: schedsim [-h] [-s id] [-i filename] [-o filename] [-l]\n");
  printf("	[-h]          : Show help information\n");
  printf("	[-s] id       : Choose the active scheduler\n");
  printf("	[-i] filename : Choose the input file\n");
  printf("	[-o] filename : Choose the output file\n");
  printf("	[-l]          : Show all schedulers\n");
}

/**
 * Function name: showScheduler
 * Description:
 *    Show all supported schedulers
 *    This includes: No., scheduler's ID, scheduler's name, scheduler's description
 **/
void showScheduler(struct SchedList scheds){
  int i=0;
  struct Scheduler* curSched=scheds.list;
  printf("No.\tId\tName\tDescription\n");
  printf("----------------------------------------------------------\n");
  for (i=0; i<scheds.nscheds; i++){
    printf("%d\t%d\t%s\t%s\n",i,curSched->id,curSched->name,curSched->description);
    curSched=curSched->next;
  }
}

/**
 * Function name: collectSupportedSchedulers
 * Description:
 *    Collect all supported schedulers
 *    Then add them to scheduler manager
 * Return value:
 *    0: If successful
 *   -1: Otherwise
 **/
static int collectSupportedSchedulers(struct SchedList *scheds)
{

  //TODO: Collect supported schedulers
  //      Initialize their default values
  //      Add them to scheds

  return 0;
}


int main(int argc, char **argv)
{
  struct Scheduler *activeSched = NULL;

  // parse options (arguments)
  parseArguments(argc, argv, &opts);

  // initialize scheduler manager
  initSchedList(&scheds);
  
  // collect all supported schedulers
  collectSupportedSchedulers(&scheds);
  
  // set active scheduler
  setActiveSched(&scheds, opts.activeSchedId);
  
  // run scheduling simulator
  if (getActiveSched(&scheds, &activeSched) == SUCCESS) {

    // get processes (tasks) from file
    activeSched->parseTasksFromFile(activeSched, opts.inFile);

    // start simulation
    activeSched->run(activeSched, opts.outFile);
  }

  freeSchedList(&scheds);
    
  return 0;
}
